<?php
/**
 * Файл для описания класса перечисления, {@link mxl_core_enum}.
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Класс описания перечисления.
 * Объект "перечисление" можно создать только путем загрузки из json-файла,
 * которая осуществляется функцией {@link mxl_core_enum::loadFromJSON()}
 * Структура json-файла определения перечисления:<br>
 * <code>
 * {
 * "name"    : "имя перечисления",
 * "caption" : "заголовок перечисления",
 * "values"  : [
 * 	{"key" : "ключ перечисления 1", "value" : "значение перечисления 1"},
 * 	{"key" : "ключ перечисления 2", "value" : "значение перечисления 2"},
 * .....
 * 	{"key" : "ключ перечисления N", "value" : "значение перечисления N"}
 * ]
 * }
 * </code>
 * 
 * Пример json-файла:
 * <code> 
 * {
 * "name"    : "yesNo",
 * "caption" : "Да нет",
 * "values"  : [
 * 	{"key" : "0", "value" : "Нет"},
 * 	{"key" : "1", "value" : "Да"}
 * ]
 * }
 * </code>
 * 
 * @package mxl_core
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
class mxl_core_enum extends mxl_core_metadata{
	/**
	 * Функция создает объект перечисление, читая json файл.
	 * Файл должен иметь имя совпадающее с именем перечисления и расширение 
	 * указанное в {@link mxl_cfg_base::$json_file_ext}.
	 * @param string $metaDataName - имя перечисления
	 * @param mxl_core_params $params - динамические параметры
	 * @return mxl_core_enum
	 */
	public static function loadFromJSON($metaDataName, mxl_core_params $params){
		$fileName = mxl_core_metadata::getJSONFileName(mxl_cfg_path::$core_enums,
				$metaDataName);
		
		$cfg = mxl_utils_json::readArrayFromFile($fileName, $params);

		$enumName    = $cfg['name'];
		$enumCaption = $cfg['caption'];
		$enumValues  = $cfg['values'];
		$enum = new mxl_core_enum($enumName, $enumCaption);
		foreach ($enumValues as $value)
			$enum->addEnumField($value['key'], $value['value']);
		return $enum;
	}

	/**
	 * Заголовок перечисления. Определяется в конструкторе.
	 * @var string
	 */
	private $caption;
	
	/**
	 * Массив хранения ключей и значений перечисления. Инициализируется в 
	 * конструкторе. Задается функцией {@link mxl_core_enum::addEnumField()}
	 * @var string
	 */
	private $listEnumFields;
	
	/**
	 * Конструктор класса перечисления.
	 * @param $name
	 * @param $caption
	 * @return mxl_core_enum
	 */
	function __construct($name, $caption){
		parent::__construct($name);		
		$this->caption        = $caption;
		$this->listEnumFields = array();
	}
	
	/**
	 * Деструктор класса перечисления. Очищает список объектов ключ-значения
	 * перечисления.
	 */
	public function __destruct(){
		unset($this->listEnumFields);
	}
	
	/**
	 * Возвращает массив ключей-значений перечисления
	 * @return array()
	 */
	public function getListEnumFields(){
		return $this->listEnumFields;
	}
	
	/**
	 * Возвращает заголовок перечисления
	 * @return string
	 */
	public function getCaption(){
		return $this->caption;
	}
	
	/**
	 * Проверяет существует ли ключ $key в перечислении 
	 * @param $key
	 * @return boolean
	 */
	public function isEnumKeyExists($key){
		return (boolean) array_key_exists($key, $this->listEnumFields);
	}
	
	/**
	 * добавляет значение в перечисление с предварительной проверкой на наличие
	 * ключа с такимже значением что и в $key

	 * @param $key - ключ перечисления
	 * @param $value - хначение перечисления
	 */
	private function addEnumField($key, $value){
		if ($this->isEnumKeyExists($key)){
			$errMess = mxl_cfg_exceptions::$core_enum_addEnumField;
			if (strpos($errMess, '#VTINS') !== false)
				$errMess = str_replace('#VTINS', $key,	$errMess);
				
			throw new Exception($errMess);
		} else
			$this->listEnumFields[$key] = $value;
	}
}
?>