<?php
/**
 * Файл для описания абстрактного класса комбобокса, {@link mxl_dict_combobx}.
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс
 */
/**
 * Абстрактный класс для хранения общей информации по комбобоксу.
 * Содержит базовые свойства и методы для комбобоксов.
 * 
 * @package mxl_dict
 * @author merlin <merlin@ersys.ru>
 * @copyright Copyright (c) 2009, ООО НПО Метарекс, {@link http://www.metarex.ru www.metarex.ru}
 */
abstract class mxl_dict_combobox extends mxl_core_metadata{
	
	/**
	 * Функция загружает комбобокс из json файла
	 * Файл должен иметь имя совпадающее с именем комбобокса и расширение 
	 * указанное в {@link mxl_cfg_base::$json_file_ext}.
	 * 
	 * Внимание!!!! Структура загружаемого json файла зависит от параметра type.
	 * Параметр может принимать значения: enum, db.
	 
	 * После определения типа комбобокса, происходит передача в загрузчик 
	 * соответствующего класса.
	 
	 * Структуры json-файлов можно посмотреть в заголовках описания классов:<br>
	 * enum в {@link mxl_dict_comboboxenum};
	 * db в {@link mxl_dict_comboboxdb}
	  
	 * @param string $metaDataName - имя комбобокса
	 * @param mxl_core_params $params - динамические параметры
	 * @return mxl_dict_combobox
	 */
	public static function loadFromJSON($metaDataName, mxl_core_params $params){
		$fileName = mxl_core_metadata::getJSONFileName(mxl_cfg_path::$dict_comboboxes,
				$metaDataName);
		
		$cfg = mxl_utils_json::readArrayFromFile($fileName, $params);

		$type = $cfg['type'];
		$combo = '';
		
		if ($type == 'enum')
			$combo = mxl_dict_comboboxenum::loadComboboxFromCfg($cfg, $combo);
		else if ($type == 'db')
			$combo = mxl_dict_comboboxdb::loadComboboxFromCfg($cfg, $combo);
		else if ($type == 'dbsql')
			$combo = mxl_dict_comboboxdbsql::loadComboboxFromCfg($cfg, $combo);

		return $combo;
	}
	
	/**
	 * Загружает базовые свойства комбобокса, которые не передаются через 
	 * конструктор но относятся к классу {@link mxl_dict_combobox}.
	 * Загрузка происходит из асоциированного масива построенного на базе
	 * json файла.
	 
	 * Загружаемые данные:<br>
	 * {@link $inputTextMode} режим фильтрации значений по введенному тексту 
	 * в комбо {@link $clearedMode} режим работы с кнопкой очистить (сбросить)
	 * значение	в комбо.
	  
	 * Стуктура в файле json:
	 * <code>
	 * "inputTextMode" : false/true - возможность ввода текста для фильтрации значений,
	 * "clearedMode"   : false/true - перевод сомбо в режим работы с кнопкой очистить
	 * </code>
	  
	 * Пример в json файле:
	 * <code>
	 * "inputTextMode" : false,
	 * "clearedMode"   : false
	 * </code>

	 * @param $cfg - ассоциированный массив, построеный на основе json файла.
	 * @param $combo - экземпляр комбобокса, который передается из потомков
	 * класса для заполнения свойств текущего класса
	 * @return mxl_dict_combobox
	 */
	public static function loadComboboxFromCfg($cfg, $combo = ''){
		$inputTextMode = $cfg['inputTextMode'];
		if ($inputTextMode == true)
			$combo->setInputTextMode(true);

		$clearedMode = $cfg['clearedMode'];
		if ($clearedMode == true)
			$combo->setClearedMode(true);
			
        if (isset($cfg['renderer'])){
            $combo->renderer = $cfg['renderer'];
        }
		
		return $combo;		
	}
	
	/**
	 * Имя поля, со значением комбобокса
	 * @var string
	 */
	private $valueField;
	/**
	 * Имя поля, которое будет отображаться
	 * @var string
	 */
	private $displayField;
	
	/**
	 * Текст, который отображается когда не выбрано значение комбобокса.
	 * @var string
	 */
	private $emptyText;
	
	/**
	 * Режим работы с фильтрацией значений по введенному тексту в комбо.
	 * @var boolean
	 */
	private $inputTextMode;
	
	/**
	 * Режим работы с кнопкой очистить(сбросить) значение комбо.
	 * @var boolean
	 */
	private $clearedMode;
	
	/**
	 * Параметр, который заполняется для варианта комбо seach.
	 * Параметр передается из клиентсокй части и содержит текст, который 
	 * был введен в комбо для отбора значений.
	 * @var string
	 */
	private $querySeach;
    
	/**
	 * Имя файла рендерера, вызывается в процессе получения данных
	 * @var string
	 */
	private $renderer;

	/**
	 * Конструктор класса {@link mxl_dict_combobx}.
	 * @param $name - имя комбобокса
	 * @param $valueField - имя поля с ключем
	 * @param $displayField - имя поля со значением
	 * @param $emptyText - текст для отображения, когда не выбрано значение
	 * @param $inputTextMode - режим фильтрации значений по введенному тексту 
	 * 		в комбо
	 * @param $clearedMode - режим работы с кнопкой очистить (сбросить) значение
	 * 		в комбо
	 * @return mxl_dict_combobx
	 */
	public function __construct($name, $valueField, $displayField, $emptyText,
			$inputTextMode = false, $clearedMode = false){
		parent::__construct($name);	
		$this->valueField    = strtolower($valueField);
		$this->displayField  = strtolower($displayField);
		
		$this->emptyText     = $emptyText;
		$this->inputTextMode = $inputTextMode;
		$this->clearedMode   = $clearedMode;
	}
	
	/**
	 * Метод для получения имени ключевого поля комбобокса.
	 * @return string
	 */
	public function getValueField(){
		return $this->valueField;
	}
	
	/**
	 * Метод для получения имени поля со значением комбобокса.
	 * @return string
	 */
	public function getDisplayField(){
		return $this->displayField;
	}
	
	/**
	 * Метод для получения текста в случае если значение комбо не задано. 
	 * @return string
	 */
	public function getEmptyText(){
		return $this->emptyText;
	}
	
	/**
	 * Метод установки значения для удаленной фильтрации значений в комбо.
	 * @param $querySeach - значение по которому будет произведена фильтрация
	 */
	public function setQuerySeach($querySeach){
		$this->querySeach = $querySeach;
	}
	/**
	 * Метод для получения значения удаленной фильтрации значений в комбо.
	 * @return string
	 */ 
	public function getQuerySeach(){
		return $this->querySeach;
	}
	
	/**
	 * Метод для перевода комбо в режим локальной фильтрации значений комбо,
	 * с фозможностью ввода текста в комбо.
	 * @param $inputTextMode - true/false - установить/снять режим
	 */
	public function setInputTextMode($inputTextMode){
		$this->inputTextMode = $inputTextMode;
	}
	/**
	 * Метод проверяющий, находится ли комбо в режиме локальной фильтрации.
	 * @return boolean
	 */
	public function isInputTextMode(){
		return $this->inputTextMode;
	}
	
	/**
	 * Метод для перевода комбо в режим работы с кнопкой очистить (сбросить).
	 * @param $clearedMode - true/false - установить/снять режим
	 */
	public function setClearedMode($clearedMode){
		$this->clearedMode = $clearedMode;
	}
	/**
	 * Метод проверяющий, находится ли комбо в режиме работы с кнопкой очистить
	 * (сбросить). 
	 * @return boolean
	 */
	public function isClearedMode(){
		return $this->clearedMode;
	}
	
	/**
	 * Возвращает имя файла рендерера
	 * @return string
	 */
	public function getRenderer(){
		return $this->renderer;
	}
	
	/**
	 * Абстрактная функция получения данных для заполнения комбобокса в 
	 * формате json и их передачи на клиентскую часть.
	 * Вывод осуществляется при помощи функции echo.
	 */
	abstract function getJSON();

	/**
	 * Абстрактная функция получения значения комбо по его ключу. 
	 * @param $startValue - ключ для которого необходимо найти значение.
	 * @return string
	 */
	abstract function getStartDisplay($startValue);
}

?>